/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.common;

import java.util.Hashtable;
import java.util.Map;

/**
 * A validation output is used to write messages while performing 
 * the validation process.  
 * 
 * @author Ali Mehregani
 */
public abstract class AbstractValidationOutput implements IValidationOutput
{	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.common.IValidationOutput#initialize(java.util.Map)
	 */
	public void initialize(Map attributes)
	{
		
	}
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.common.IValidationOutput#reportMessage(org.eclipse.cosmos.rm.validation.internal.common.AbstractValidationOutput.ValidationMessage)
	 */
	public abstract void reportMessage(ValidationMessage validationMessage);
	
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.common.IValidationOutput#reportError(java.lang.String)
	 */
	public void reportError(String message)
	{		
		reportMessage(ValidationMessage.SEVERITY_ERROR, message);
	}
	
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.common.IValidationOutput#reportWarning(java.lang.String)
	 */
	public void reportWarning(String message)
	{		
		reportMessage(ValidationMessage.SEVERITY_WARNING, message);
	}
	
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.common.IValidationOutput#reportInfo(java.lang.String)
	 */
	public void reportInfo(String message)
	{		
		reportMessage(ValidationMessage.SEVERITY_INFO, message);
	}
	
	
	private void reportMessage(int severity, String message)
	{
		ValidationMessage verificationMsg = new ValidationMessage();
		verificationMsg.setAttribute(ValidationMessage.ATTRIBUTE_SEVERITY, severity);
		verificationMsg.setAttribute(ValidationMessage.ATTRIBUTE_MESSAGE, message);
		reportMessage(verificationMsg);
	}
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.common.IValidationOutput#close()
	 */
	public void close()
	{
		
	}
	
	
	/**
	 * Represents a verification message, which is simply described as a set
	 * of unstructured attributes.
	 * 
	 * @author Ali Mehregani
	 */
	public static class ValidationMessage
	{
		/**
		 * Indicates a severity of info
		 */
		public static final int SEVERITY_INFO = 0;
		
		/**
		 * Indicates a severity of warning
		 */
		public static final int SEVERITY_WARNING = 1;
		
		/**
		 * Indicates a severity of error
		 */
		public static final int SEVERITY_ERROR = 2;
		
		/**
		 * The message attribute.  The value of this attribue is expected
		 * to be a {@link String}
		 */
		public static final String ATTRIBUTE_MESSAGE = "attribute.message";
		
		/**
		 * The severity attribute.  The value of this attribute is expected
		 * to be one of: <br/>
		 * SEVERITY_INFO <br/>
		 * SEVERITY_WARNING <br/>
		 * SEVERITY_ERROR <br/>
		 */
		public static final String ATTRIBUTE_SEVERITY = "attribute.severity";
		
		/**
		 * An object of type IResource indicating the resource associated with this message
		 */
		public static final String ATTRIBUTE_RESOURCE = "attribute.resource";
		
		/**
		 * An integer indicating the line number associated with this message
		 */
		public static final String ATTRIBUTE_LINE_NUMBER = "attribute.line.number";
		
		
		/**
		 * The attributes of the message
		 */
		private Map attributes; 
				
		
		public ValidationMessage()
		{
			attributes = new Hashtable();
		}
				
		/**
		 * Set a string attribute
		 * 
		 * @param key The key 
		 * @param value The value
		 */
		public void setAttribute(String key, String value)
		{
			setAttribute(key, (Object)value);
		}
		
		/**
		 * Set an integer attribute
		 * 
		 * @param key The key 
		 * @param value The value
		 */
		public void setAttribute(String key, int value)
		{
			attributes.put(key, new Integer(value));
		}		
		
		/**
		 * Set a boolean attribute
		 * 
		 * @param key The key 
		 * @param value The value
		 */
		public void setAttribute(String key, boolean value)
		{
			attributes.put(key, new Boolean(value));
		}		
		
		/**
		 * Set an object attribute
		 * 
		 * @param key The key 
		 * @param value The value
		 */
		public void setAttribute(String key, Object value)
		{
			attributes.put(key, value);	
		}
		
		
		/**
		 * Retrieve an integer attribute
		 * 
		 * @param key The key 
		 * @param defaultValue The default value
		 */
		public int getAttribute(String key, int defaultValue)
		{
			Object value = attributes.get(key);
			if (value instanceof Integer)
				return ((Integer)value).intValue();
			return defaultValue;
		}		
		
		/**
		 * Retrieve a boolean attribute
		 * 
		 * @param key The key 
		 * @param defaultValue The default value
		 */
		public boolean getAttribute(String key, boolean defaultValue)
		{
			Object value = attributes.get(key);
			if (value instanceof Boolean)
				return ((Boolean)value).booleanValue();
			return defaultValue;
		}
				
		/**
		 * Retrieve a string attribute
		 * 
		 * @param key The key 
		 * @param defaultValue The default value
		 */
		public String getAttribute(String key, String defaultValue)
		{
			Object value = attributes.get(key);
			if (value instanceof String)
				return (String)value;
			return defaultValue;
		}
		
		/**
		 * Retrieve an object attribute
		 * 
		 * @param key The key 
		 */
		public Object getAttribute(String key)
		{
			return attributes.get(key);
		}
	}
}
