/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.core;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.eclipse.cosmos.rm.validation.internal.common.AbstractValidationOutput;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DataBuilderRegistry;
import org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Implements a custom SAX Default Handler that will delegate ContentHandler calls to IDataBuilder instances.
 *  
 * @author sleeloy
 *
 */
public class DatastructureBuilderImpl extends DefaultHandler implements IDataStructureBuilder{
    protected SAXParser saxParser;
    /**
     * List of IDataBuilder instances
     */
    protected Collection dataStructureBuilder;    
    protected AbstractValidationOutput validationLogger;
    /**
     * custom xml schmea validator
     */
    protected IXMLValidator xmlValidation;
    protected String xml;
    protected boolean validateXML;
    protected byte phase = ISMLConstants.START_PHASE;
    protected Locator locator;
    
	public DatastructureBuilderImpl() {
		super();
		dataStructureBuilder = DataBuilderRegistry.instance().getDataStructureBuilders();

	}
	
	/**
	 * TODO: xml instance should be passed in as an attribute
	 */
	public void setXML(Object xml){
		if (xml instanceof String)
			this.xml = (String)xml;		
	}
	
	public void initialize(Map validationAttribute) {
		validationLogger = (AbstractValidationOutput)validationAttribute.get(ATTRIBUTE_OUTPUT);
		Object validationClass = (validationAttribute.get(MainValidator.ATTRIBUTE_VALDIATION_XML));
		//if no validation class then we should do no XML schema validation
		validateXML = true;
		if (validationClass == null){
			validateXML = false;
		}
		else if (validationClass.equals(IXMLValidator.DEFAULT_VALIDATION_XML_LITERAL)){
			//should validate and use the defalut validator
			xmlValidation = null;			
		}
		else{
			//should use the custom xml schema validator
			IValidator[] validation = ValidationFactory.createValidator(IXMLValidator.class, validationClass);
			xmlValidation = validation.length > 0 ? (IXMLValidator)validation[0] : null;
			xmlValidation.initialize(validationAttribute);
		}
		setSchema((String)validationAttribute.get(ATTRIBUTE_SCHEMA));
		
	}	
	
	/**
	 * Configures the default handler and associates the schema with the SAX Parser
	 * @param xsd file location of the schema file to validate
	 */
	protected void setSchema(String xsd) {
       try {   	
    	   //if there is no schema then we should not validate
   	    	if (xsd == null)
   	    		validateXML = false;
    	   
    	    SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
    	    saxParserFactory.setNamespaceAware(true);
    	    saxParserFactory.setValidating(validateXML);

			saxParserFactory.setFeature("http://xml.org/sax/features/validation", validateXML);
			saxParserFactory.setFeature("http://xml.org/sax/features/namespaces", true);
			saxParserFactory.setFeature("http://xml.org/sax/features/namespace-prefixes", true);

			SAXParser newSaxParser = saxParserFactory.newSAXParser();

			if (xsd != null){
				newSaxParser.setProperty("http://java.sun.com/xml/jaxp/properties/schemaLanguage","http://www.w3.org/2001/XMLSchema");
				newSaxParser.setProperty("http://java.sun.com/xml/jaxp/properties/schemaSource", new FileInputStream(new File(xsd)));
			}
			saxParser = newSaxParser;
		} catch (ParserConfigurationException e) {
		      fatalError(e);   
		} catch (IOException e) {
		      fatalError(e);   
		} catch (SAXNotRecognizedException e) {
			//Some parsers cannot validate the xsd like crimson.  Therefore turn off validation.
			  e.printStackTrace();
		      fatalError(e);
		} catch (SAXNotSupportedException e) {
			//Some parsers cannot validate the xsd like crimson.  Therefore turn off validation.
			  e.printStackTrace();
		      fatalError(e);
		} catch (SAXException e) {
		      fatalError(e);       	    
		}
	}
	  protected void fatalError(ParserConfigurationException exception)
	  {
		  validationLogger.reportError(exception.getLocalizedMessage());
	  }

	  public void fatalError(SAXException exception)
	  {
		  validationLogger.reportError(exception.getLocalizedMessage());
	  }

	  public void fatalError(SAXParseException exception)
	  {
		  validationLogger.reportError(exception.getLocalizedMessage());
	  }

	  public void error(SAXParseException exception)
	  {
  	    validationLogger.reportError(exception.getLocalizedMessage() + " \nLine:" + exception.getLineNumber() + " \nColumn:"+exception.getColumnNumber());
	  }
	   
	  protected void fatalError(IOException exception)
	  {
		  validationLogger.reportError(exception.getLocalizedMessage());
	  }
	  
	  public boolean validate()
	  {
	    try
	    {
	      saxParser.parse(new InputSource(new FileInputStream(new File(xml))), this);
	      return true;
	    }
	    catch (IOException exception)
	    {
	      fatalError(exception);
	    }
	    catch (SAXException exception)
	    {
	       fatalError(exception);
	    }
	    
	    return false;
	  }
	  

	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
		boolean startGenicsPhase = false;
		boolean startPhenicsPhase = false;
		if (phase != ISMLConstants.PHENICS_PHASE){
	 		if ((phase != ISMLConstants.GENICS_PHASE)&& (ISMLConstants.GENICS_ELEMENT.equals(localName))){
	 			phase = ISMLConstants.GENICS_PHASE;
	 			startGenicsPhase = true;
	 		}
	 		else if (ISMLConstants.PHENICS_ELEMENT.equals(localName)){
	 			phase = ISMLConstants.PHENICS_PHASE; 			
	 			startPhenicsPhase = true;
	 		}
		}
		//build datatructures
		Iterator iter = dataStructureBuilder.iterator();
		while (iter.hasNext()){			
			IDataBuilder dataBuilder = (IDataBuilder)iter.next();
			if (startGenicsPhase)
				dataBuilder.startGenics();
			if (startPhenicsPhase)
				dataBuilder.startPhenics();
			if ((dataBuilder.getPhase() == phase) || (dataBuilder.getPhase() == ISMLConstants.GENICS_PHENICS_PHASE && (phase == ISMLConstants.PHENICS_PHASE || phase == ISMLConstants.GENICS_PHASE)))
				dataBuilder.startElement(uri, localName, qName, attributes);			
		}
		
		if (xmlValidation != null)
			xmlValidation.startElement(uri, localName, qName, attributes);
	}

	public void endElement(String uri, String localName, String qName) throws SAXException {
		boolean endGenicsPhase = false;
		boolean endPhenicsPhase = false;
 		if ((phase == ISMLConstants.GENICS_PHASE) && (ISMLConstants.GENICS_ELEMENT.equals(localName))){
 			phase = ISMLConstants.GENICS_PHENICS_PHASE;
 			endGenicsPhase = true;
 		}
 		else if ((phase == ISMLConstants.PHENICS_PHASE) && (ISMLConstants.PHENICS_ELEMENT.equals(localName))){
 			phase = ISMLConstants.GENICS_PHENICS_PHASE; 	
 			endPhenicsPhase = true;
 		}
		//build datatructures
		Iterator iter = dataStructureBuilder.iterator();
		while (iter.hasNext()){			
			IDataBuilder dataBuilder = (IDataBuilder)iter.next();
			if ((dataBuilder.getPhase() == phase) || (dataBuilder.getPhase() == ISMLConstants.GENICS_PHENICS_PHASE && (phase == ISMLConstants.PHENICS_PHASE || phase == ISMLConstants.GENICS_PHASE)))
				dataBuilder.endElement(uri, localName, qName);
			if (endGenicsPhase)
				dataBuilder.endGenics();
			if (endPhenicsPhase)
				dataBuilder.endPhenics();
		}
		
		if (xmlValidation != null)
			xmlValidation.endElement(uri, localName, qName);
	}

	public void characters(char[] ch, int start, int len) throws SAXException {
		//build datatructures
		Iterator iter = dataStructureBuilder.iterator();
		while (iter.hasNext()){			
			IDataBuilder dataBuilder = (IDataBuilder)iter.next();
			if ((dataBuilder.getPhase() == phase) || (dataBuilder.getPhase() == ISMLConstants.GENICS_PHENICS_PHASE && (phase == ISMLConstants.PHENICS_PHASE || phase == ISMLConstants.GENICS_PHASE)))
				dataBuilder.characters(ch, start, len);
		}
		
		if (xmlValidation != null)
			xmlValidation.characters(ch, start, len);	
	}

	public void endDocument() throws SAXException {
		//build datatructures
		Iterator iter = dataStructureBuilder.iterator();
		while (iter.hasNext()){			
			IDataBuilder dataBuilder = (IDataBuilder)iter.next();
			if ((dataBuilder.getPhase() == phase) || (dataBuilder.getPhase() == ISMLConstants.GENICS_PHENICS_PHASE && (phase == ISMLConstants.PHENICS_PHASE || phase == ISMLConstants.GENICS_PHASE)))
				dataBuilder.endDocument();
		}
		
		if (xmlValidation != null)
			xmlValidation.endDocument();	
	}

	public void endPrefixMapping(String arg0) throws SAXException {
		//build datatructures
		Iterator iter = dataStructureBuilder.iterator();
		while (iter.hasNext()){			
			IDataBuilder dataBuilder = (IDataBuilder)iter.next();
			if ((dataBuilder.getPhase() == phase) || (dataBuilder.getPhase() == ISMLConstants.GENICS_PHENICS_PHASE && (phase == ISMLConstants.PHENICS_PHASE || phase == ISMLConstants.GENICS_PHASE)))
				dataBuilder.endPrefixMapping(arg0);
		}
		
		if (xmlValidation != null)
			xmlValidation.endPrefixMapping(arg0);	
	}

	public void unparsedEntityDecl(String arg0, String arg1, String arg2, String arg3) throws SAXException {
		// TODO Auto-generated method stub
		super.unparsedEntityDecl(arg0, arg1, arg2, arg3);
	}

	public void ignorableWhitespace(char[] arg0, int arg1, int arg2) throws SAXException {
		Iterator iter = dataStructureBuilder.iterator();
		while (iter.hasNext()){			
			IDataBuilder dataBuilder = (IDataBuilder)iter.next();
			if ((dataBuilder.getPhase() == phase) || (dataBuilder.getPhase() == ISMLConstants.GENICS_PHENICS_PHASE && (phase == ISMLConstants.PHENICS_PHASE || phase == ISMLConstants.GENICS_PHASE)))
				dataBuilder.ignorableWhitespace(arg0, arg1, arg2);
		}
		
		if (xmlValidation != null)
			xmlValidation.ignorableWhitespace(arg0, arg1, arg2);	
	}

	public void processingInstruction(String arg0, String arg1) throws SAXException {
		//build datatructures
		Iterator iter = dataStructureBuilder.iterator();
		while (iter.hasNext()){			
			IDataBuilder dataBuilder = (IDataBuilder)iter.next();
			if ((dataBuilder.getPhase() == phase) || (dataBuilder.getPhase() == ISMLConstants.GENICS_PHENICS_PHASE && (phase == ISMLConstants.PHENICS_PHASE || phase == ISMLConstants.GENICS_PHASE)))
				dataBuilder.processingInstruction(arg0, arg1);
		}
		
		if (xmlValidation != null)
			xmlValidation.processingInstruction(arg0, arg1);;	
	}

	public void setDocumentLocator(Locator locator) {
		this.locator = locator;
		//build datatructures
		Iterator iter = dataStructureBuilder.iterator();
		while (iter.hasNext()){			
			IDataBuilder dataBuilder = (IDataBuilder)iter.next();
			dataBuilder.setDocumentLocator(locator);
		}
		
		if (xmlValidation != null)
			xmlValidation.setDocumentLocator(locator);	
	}

	public void skippedEntity(String arg0) throws SAXException {
		//build datatructures
		Iterator iter = dataStructureBuilder.iterator();
		while (iter.hasNext()){			
			IDataBuilder dataBuilder = (IDataBuilder)iter.next();
			if ((dataBuilder.getPhase() == phase) || (dataBuilder.getPhase() == ISMLConstants.GENICS_PHENICS_PHASE && (phase == ISMLConstants.PHENICS_PHASE || phase == ISMLConstants.GENICS_PHASE)))
				dataBuilder.skippedEntity(arg0);
		}
		
		if (xmlValidation != null)
			xmlValidation.skippedEntity(arg0);;	
	}

	public void startDocument() throws SAXException {
		//build datatructures
		Iterator iter = dataStructureBuilder.iterator();
		while (iter.hasNext()){			
			IDataBuilder dataBuilder = (IDataBuilder)iter.next();
			if ((dataBuilder.getPhase() == phase) || (dataBuilder.getPhase() == ISMLConstants.GENICS_PHENICS_PHASE && (phase == ISMLConstants.PHENICS_PHASE || phase == ISMLConstants.GENICS_PHASE)))
				dataBuilder.startDocument();
		}
		
		if (xmlValidation != null)
			xmlValidation.startDocument();
	}

	public void startPrefixMapping(String arg0, String arg1) throws SAXException {
		//build datatructures
		Iterator iter = dataStructureBuilder.iterator();
		while (iter.hasNext()){			
			IDataBuilder dataBuilder = (IDataBuilder)iter.next();
			if ((dataBuilder.getPhase() == phase) || (dataBuilder.getPhase() == ISMLConstants.GENICS_PHENICS_PHASE && (phase == ISMLConstants.PHENICS_PHASE || phase == ISMLConstants.GENICS_PHASE)))
				dataBuilder.startPrefixMapping(arg0, arg1);
		}
		
		if (xmlValidation != null)
			xmlValidation.startPrefixMapping(arg0, arg1);;	
	}
}
