/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.databuilders;

import org.xml.sax.ContentHandler;


/**
 * This interface is used to represent a data builder which will construct
 * a data structure while an SML document is being parsed. 
 * 
 * @author sleeloy
 * @author Ali Mehregani
 */
public interface IDataBuilder extends ContentHandler
{
	/**
	 * Returns the data structure.  The type of the structure must be specified as
	 * part of the specification of the class that implements this interface.
	 * 
	 * @return The data structure
	 */
	public Object getDataStructure();

	
	/**
	 * This method will be called when the parser starts parsing the 
	 * genics part of the document 
	 */
	public void startGenics();
	
	
	/**
	 * This method will be called when the parser starts parsing the phenics 
	 * part of the document 
	 */
	public void startPhenics();
	
	
	/**
	 * This method will be called when the parser ends parsing the genics 
	 * part of the document 
	 */
	public void endGenics();
	
	
	/**
	 * This method will be called when the parser ends parsing the phenics part 
	 * of the document 
	 */
	public void endPhenics();
	
	
	/**
	 * Specifies the phase that this data builder is active during parsing.  The parser parses the sml 
	 * document in two parts: Genics and Phenics phases.  The valid values are as follows:
	 * <pre>
	 * ISMLConstants.GENICS_PHASE -databuilder is active during the parsing of the genics elements
	 * ISMLConstants.PHENICS_PHASE -databuilder is active during the parsing of the phenics elements
	 * ISMLConstants.GENICS_PHENICS_PHASE - databuilder is active during the parsing of the genics and phenics elements
	 * </pre>
	 *    
	 * @return a phase that this databuilder will be active
	 */
	public byte getPhase(); 
	
	
	/**
	 * Returns true if this structure is valid.  An error message is usually
	 * set if this method returns false.
	 */
	public boolean isStructureValid();
	
	
	/**
	 * Sets the structure validity
	 *  
	 * @param validity The validity of the structure
	 */
	public void setStructureValidity(boolean validity);
	
	
	/**
	 * Sets an error message in case the structure is not valid
	 * 
	 * @param error The error message
	 */
	public void setErrorMessage(String error);

	
	/**
	 * Returns the error message set.  This value can be null if no error
	 * message has been set.
	 * 
	 * @return The error message
	 */
	public String getErrorMessage();
}
