/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.databuilders;

import org.eclipse.cosmos.rm.validation.internal.SMLActivator;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.cosmos.rm.validation.internal.common.SMLCommonUtil;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * This databuilder constructs an SMLIFIdentity object which will include
 * the content that is included in the identity section of an SML-IF 
 * document.
 * 
 * @author Ali Mehregani
 */
public class IdentityDataBuilder extends AbstractDataBuilder
{
	/**
	 * The ID of this builder
	 */
	public static final String ID = SMLActivator.PLUGIN_ID + ".IdentityDataBuilder";
	
	
	/**
	 * The sml-if identity object
	 */
	private SMLIFIdentity smlIfIdentity;
	
	/**
	 * Flags used to indicate the start of an element
	 */
	private boolean identityElementHit;
	private boolean nameElementHit;
	private boolean descriptionElementHit;
	private boolean baseURIElementHit;
	
	/**
	 * The name stored in the identity section
	 */
	private String name;

	/**
	 * The description stored in the identity section
	 */
	private String description;
	
	/**
	 * The base URI stored in the identity section
	 */
	private String  baseURI;
	
	/**
	 * Constructor
	 */
	public IdentityDataBuilder()
	{
		smlIfIdentity = new SMLIFIdentity();		
	}

	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.AbstractDataBuilder#getPhase()
	 */
	public byte getPhase()
	{
		return ISMLConstants.START_PHASE;
	}
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.AbstractDataBuilder#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
	{
		elementNotification (uri, localName, true);
	}
	
	
	/**
	 * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
	 */
	public void characters(char[] ch, int start, int length) throws SAXException
	{
		if (nameElementHit)
		{
			name += new String(ch, start, length);
		}
		else if (descriptionElementHit)
		{
			description += new String(ch, start, length);
		}
		else if (baseURIElementHit)
		{
			baseURI += new String (ch, start, length); 
		}
	}
	
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.AbstractDataBuilder#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void endElement(String uri, String localName, String qName) throws SAXException
	{
		elementNotification (uri, localName, false);
	}
	
	
	private void elementNotification (String uri, String localName, boolean start)
	{
		if (!ISMLConstants.SMLIF_URI.equals(uri))
			return;
			
		if (ISMLConstants.IDENTITY_ELEMENT.equals(localName))
		{
			identityElementHit = start;
		}
		else if (identityElementHit)
		{
			if (ISMLConstants.NAME_ATTRIBUTE.equals(localName))
			{
				nameElementHit = start;
				if (!start)
					smlIfIdentity.setName(name);
			}
			else if (ISMLConstants.DESCRIPTION_ELEMENT.equals(localName))
			{
				descriptionElementHit = start;
				if (!start)
					smlIfIdentity.setDescription(description);
			}
			else if (ISMLConstants.BASE_URI_ELEMENT.equals(localName))
			{
				baseURIElementHit = start;
				if (!start)
					smlIfIdentity.setBaseURI(baseURI);
			}
		}
	}
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder#getDataStructure()
	 */
	public Object getDataStructure()
	{
		return smlIfIdentity;
	}
	
	
	
	/**
	 * Stores the data included in the identity section of the
	 * SML-IF document
	 * 
	 * @author Ali Mehregani
	 */
	public static class SMLIFIdentity
	{
		private String name;
		private String description;
		private String baseURI;
		
		/**
		 * @return the baseURI
		 */
		public String getBaseURI()
		{
			return baseURI;
		}
		/**
		 * @param baseURI the baseURI to set
		 */
		public void setBaseURI(String baseURI)
		{
			this.baseURI = baseURI;
		}
		/**
		 * @return the description
		 */
		public String getDescription()
		{
			return description;
		}
		/**
		 * @param description the description to set
		 */
		public void setDescription(String description)
		{
			this.description = description;
		}
		/**
		 * @return the name
		 */
		public String getName()
		{
			return name;
		}
		/**
		 * @param name the name to set
		 */
		public void setName(String name)
		{
			this.name = name;
		}		
	}


	/**
	 * Retrieves and returns the base URI
	 * @return the base URI
	 */
	public static String retrieveBaseURI()
	{		
		SMLIFIdentity smlIdentity = (SMLIFIdentity)SMLCommonUtil.retrieveDataStructure(ID);
		return smlIdentity == null ? null : smlIdentity.getBaseURI();
	}
}
